using System;
using System.Net;
using System.IO;
using System.Text;
using System.Threading;
using System.Diagnostics;
using HumanityServices;


namespace HumanityServices
{

	/// <summary>
	/// Class: AsynchWebRequest
	/// </summary>
	public class AsynchWebRequest
	{
		/// <summary>
		/// threadCompletionEvent
		/// </summary>
		public static ManualResetEvent threadCompletionEvent= new ManualResetEvent(false);

		/// <summary>
		/// LogRequestEvents
		/// </summary>
		public static bool LogRequestEvents;

		const int BUFFER_SIZE = 1024;
		const int DefaultTimeout = 60000; // in ms. 1 minute by default...
 
		// Abort the request if the timer fires.
		private static void TimeoutCallback(object state, bool timedOut) 
		{ 
			if (timedOut) 
			{
				HttpWebRequest request = state as HttpWebRequest;
				if (request != null) 
				{
					request.Abort();
					EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Timed Out Request. The last request took longer than 10 seconds to respond. ");
				}
			}
		}

		/// <summary>
		/// Class: AsynchWebRequest
		/// </summary>
		/// <param name="URLToRequest">URL of web site or page to access</param>
		/// <param name="ShouldLog">True if logging is enabled;otherwise, false</param>
		public AsynchWebRequest(String URLToRequest, bool ShouldLog)
		{
			//save logging state;
			LogRequestEvents = ShouldLog;

			try
			{
				if(ShouldLog == true)
					EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Accessing URL: " + URLToRequest);

				if(URLToRequest != null)
				{
					// Create a HttpWebRequest object to the desired URL. 
					HttpWebRequest targetWebRequest= (HttpWebRequest)WebRequest.Create(URLToRequest);

					if(targetWebRequest != null)
					{
						if(ShouldLog == true)
							EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Web Request created successfully for URL: " + URLToRequest);

	
						// Create an instance of the CurrentRequestState and assign the previous targetWebRequest
						// object to it's request field.  
						CurrentRequestState targetRequestState = new CurrentRequestState();
						if(targetRequestState != null)
						{
							targetRequestState.request = targetWebRequest;
						}


						// Begin an asynchronous request; register its callback method
						IAsyncResult result=
							(IAsyncResult) targetWebRequest.BeginGetResponse(new AsyncCallback(asynchResponseCallback),targetRequestState);

						if(ShouldLog == true)
							EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Implementing a 1 minute timeout for this request for URL: " + URLToRequest);

						// this line impliments the timeout, if there is a timeout, the callback fires and the request becomes aborted
						ThreadPool.RegisterWaitForSingleObject (result.AsyncWaitHandle, new WaitOrTimerCallback(TimeoutCallback), targetWebRequest, DefaultTimeout, true);

						// The response came in the allowed time. The work processing will happen in the 
						// callback function.
						threadCompletionEvent.WaitOne();

						if(ShouldLog == true)
							EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Releasing Resources for URL: " + URLToRequest + " and closing the connection.");
    
						// Release the HttpWebResponse resource.
						if(targetRequestState != null)
						{
							targetRequestState.response.Close();
						}

					}
					else
					{
						if(ShouldLog == true)
							EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: Web Request was not created successfully for URL: " + URLToRequest);

					}
				}
				else
				{
					if(ShouldLog == true)
						EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest: NULL URL value was passed to function.");

				}
    
			}
			catch
			{
				//ignore any errors...
			}
		}//end of constructor method

		private static void asynchResponseCallback(IAsyncResult asynchronousResult)
		{  
			try
			{
				if(LogRequestEvents == true)
					EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest.asynchResponseCallback: Callback event triggered. Starting asynchronous read now ");

				// Mark the current request as asynchronous
				CurrentRequestState targetRequestState=(CurrentRequestState) asynchronousResult.AsyncState;
				HttpWebRequest  targetWebRequest=targetRequestState.request;
				targetRequestState.response = (HttpWebResponse) targetWebRequest.EndGetResponse(asynchronousResult);
      
				// Read the returned HttpResponse into the stream; save current state info
				Stream responseStream = targetRequestState.response.GetResponseStream();
				targetRequestState.asynchResponseStream=responseStream;
      
				// Start reading the target HTML page into the temporary byte array
				IAsyncResult asynchronousInputRead = responseStream.BeginRead(targetRequestState.workReadBuffer, 0, BUFFER_SIZE, new AsyncCallback(asynchCallBack), targetRequestState);
				return;
			}
			catch(Exception ex)
			{

				EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest.asynchResponseCallback: Error in the Response CallBack " + ex.Message);

			}
			threadCompletionEvent.Set();
		}


		private static  void asynchCallBack(IAsyncResult asyncResult)
		{
			try
			{
                //mark the current request as asynchronous
				CurrentRequestState targetRequestState = (CurrentRequestState)asyncResult.AsyncState;
				Stream responseStream = targetRequestState.asynchResponseStream;
				int read = responseStream.EndRead( asyncResult );
				// Read the HTML page and then do nothing with it...
				if (read > 0)
				{
					targetRequestState.requestData.Append(Encoding.ASCII.GetString(targetRequestState.workReadBuffer, 0, read));
					IAsyncResult asynchronousResult = responseStream.BeginRead( targetRequestState.workReadBuffer, 0, BUFFER_SIZE, new AsyncCallback(asynchCallBack), targetRequestState);
					return;
				}
				else
				{
					
					if(targetRequestState.requestData.Length>1)
					{
						string stringContent;
						stringContent = targetRequestState.requestData.ToString();
					}
					      
					if(LogRequestEvents == true)
						EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest.asynchCallBack: Asynchronous Read completed. Closing response stream now ");

					responseStream.Close();
				}

			}
			catch(Exception ex)
			{
                EventLog.WriteEntry("HumanityTHUNDER", "AsynchWebRequest.asynchCallBack: Error in the Asynchronous Request READ CallBack " + ex.Message);

			}
			threadCompletionEvent.Set();

		}

	}//end of class

    /// <summary>
    /// Class: CurrentRequestState
    /// 2005,2006 Advance Guard Technologies - All Rights reserved
    /// This class provides a mechanism for maintaining the current state of
    /// the asynchronous request.
    /// </summary>
    public class CurrentRequestState
    {
        const int BUFFER_SIZE = 1024;

        /// <summary>
        /// StringBuilder: requestData
        /// 2005,2006 Advance Guard Technologies - All Rights reserved
        /// 
        /// </summary>
        public StringBuilder requestData;

        /// <summary>
        /// workReadBuffer
        /// </summary>
        public byte[] workReadBuffer;

        /// <summary>
        /// request
        /// </summary>
        public HttpWebRequest request;

        /// <summary>
        /// response
        /// </summary>
        public HttpWebResponse response;

        /// <summary>
        /// asynchResponseStream
        /// </summary>
        public Stream asynchResponseStream;


        /// <summary>
        /// Constructor: CurrentRequestState
        /// Initialize class member variables to a "known" state
        /// </summary>
        public CurrentRequestState()
        {
            //set to "good" default values
            workReadBuffer = new byte[BUFFER_SIZE];
            requestData = new StringBuilder("");
            request = null;
            asynchResponseStream = null;
        }

    }//end of class definition

}//end of namespace
